@@ -74,15 +74,13 @@
         (setq node (treesit-node-at (dwim-coder-preceding-point)))
         (when (member (treesit-node-type node) '("=" "enum"))
           (cl-return-from dwim-coder-c-skip-semi nil))
-        (setq node (treesit-node-parent (treesit-node-at (dwim-coder-preceding-point))))
+        (setq node (treesit-node-parent (treesit-node-parent (treesit-node-at (dwim-coder-preceding-point)))))
         (when (equal (treesit-node-type node) "struct_specifier")
           (cl-return-from dwim-coder-c-skip-semi nil)))
         (setq skip-semi t))
     (when (eq (preceding-char) ?\))
       (save-excursion
-        (backward-char)
         (backward-sexp)
-        (backward-char)
         (setq node (treesit-node-at (dwim-coder-preceding-point)))
         (when (member (treesit-node-type node) '("if" "for"))
           (cl-return-from dwim-coder-c-skip-semi t))
@@ -127,39 +125,37 @@
               (treesit-node-text node t))))))
 
 (defun dwim-coder-c-point-around-defun-decl ()
-  (let ((node nil)
+  (let ((func-node nil)
+        (param-node nil)
+        (decl-node nil)
+        (node nil)
         (func-start nil)
         (func-name-start nil)
         (func-arg-start nil)
         (func-arg-end nil))
-    (setq node (treesit-node-top-level
-                (treesit-node-at (point)) "^function_definition$"))
-    (if node
-        (progn
-          (setq func-start (treesit-node-start node))
-          (setq node (treesit-node-child-by-field-name node "declarator")))
-      (setq node (treesit-node-top-level (treesit-node-at (point)) "^declaration$"))
-      (setq func-start (treesit-node-start node)))
-    (if node
-        (setq node (treesit-search-subtree node "function_declarator"))
-      ;; If we didn't get the declarator so far, we might be in a declaration that's
-      ;; not a definition.
-      ;; fixme: func-start points to the start of function name, not function type name
-      (setq node (treesit-node-top-level
-                  (treesit-node-at (point)) "^function_declarator$"))
-      (setq func-start (treesit-node-start node)))
-    (when node
-      (setq func-name-start (treesit-node-start node))
-      (setq node (treesit-filter-child
-                  node (lambda (n) (equal (treesit-node-type n) "parameter_list"))))
-      (when node
-        (setq node (car node))
-        (setq func-arg-start (treesit-node-start node))
-        (setq func-arg-end (treesit-node-end node))))
-    (if (and func-start func-arg-start func-arg-end
-             (>= (point) func-start)
-             (<= (point) func-arg-end))
-        (list func-start func-name-start func-arg-start func-arg-end))))
+    (setq func-node (treesit-node-top-level
+                     (treesit-node-at (point)) "^function_definition$"))
+    (if func-node (setq func-start (treesit-node-start func-node)))
+
+    (setq func-node (treesit-node-top-level
+                     (treesit-node-at (dwim-coder-preceding-point)) "^function_declarator$"))
+    (if func-node (setq func-name-start (treesit-node-start func-node)))
+
+    (when (and func-name-start (not func-start))
+      (setq func-node (treesit-node-top-level
+                       (treesit-node-at (point)) "^declaration$"))
+      ;; Incomplete functions may result in ERROR, handle that too
+      (unless func-node
+        (setq func-node (treesit-node-top-level
+                         (treesit-node-at (point)) "^ERROR$")))
+      (if func-node (setq func-start (treesit-node-start func-node))))
+    (setq param-node (treesit-node-top-level
+                     (treesit-node-at (dwim-coder-preceding-point)) "^parameter_list$"))
+
+    (when (and func-start func-name-start param-node)
+      (setq func-arg-start (treesit-node-start param-node))
+      (setq func-arg-end (treesit-node-end param-node))
+      (list func-start func-name-start func-arg-start func-arg-end))))
 
 (defun dwim-coder-c-in-include-fname ()
   (let ((node (treesit-node-at (point))))
@@ -372,7 +368,6 @@
                  (eglot-current-server))
         (ignore-errors (eglot-format (1- (point)) (1+ (point)))
                        ;; eglot doesn't seem to indent closing } sometimes, so do it manually
-                       (forward-char)
                        (indent-according-to-mode)
                        (skip-chars-backward "[:space:]\n")
                        ;; Insert a new blank line
@@ -482,7 +477,8 @@
       (dwim-coder-insert-interactive ?\( t)
       t))))
 
-(defun dwim-coder-c-dwim-brace ()
+
+(cl-defun dwim-coder-c-dwim-brace ()
   (let ((value nil))
     (cond
      ;; Handle '{' in function declaration
@@ -504,14 +500,29 @@
       (goto-char (nth 3 value))
       (if (eq (following-char) ?\;)
           (delete-char 1))
+
+      ;; If the function is already followed by a {, skip to that
+      (when (save-excursion
+              (skip-chars-forward "[ \n\t]" (+ 1 (point)))
+              (eq (following-char) ?{))
+        (when (eq dwim-coder-c-sub-style 'gnome)
+          (align (nth 2 value) (nth 3 value)))
+        (skip-chars-forward "[ \n\t{]" (+ 2 (point)))
+        (skip-chars-forward "[\n]" (+ 1 (point)))
+        (indent-according-to-mode)
+        (if (eq (following-char) ?})
+            (backward-char))
+        (cl-return-from dwim-coder-c-dwim-brace t))
+
       ;; insert temporary code so that eglot-format will format braces right
       (insert "{dwim();}")
-      (goto-char (nth 2 value))
+      (goto-char (nth 3 value))
       (setq value (dwim-coder-c-point-around-defun-decl))
       (when (and (fboundp 'eglot-current-server)
                  (eglot-current-server))
         (ignore-errors (eglot-format (nth 0 value) (+ (length "{dwim();}") (nth 3 value)))))
       (goto-char (nth 2 value))
+      (forward-char)
       (setq value (dwim-coder-c-point-around-defun-decl))
       ;; Align arguments in gnome style
       (when (eq dwim-coder-c-sub-style 'gnome)
